/* libipt_TPROXYRST.c -  */

/* Copyright (c) 2010, TeamF1, Inc. */

/*
modification history
--------------------
01a,21jan10,kri  written
*/

/* Shared library add-on to iptables to add TPROXYRST target support. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <iptables.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include "../include/linux/netfilter_ipv4/ipt_TPROXYRST.h"


/* Function which prints out usage message. */
static void
tproxyrst_target_help(void)
{
	printf(
"TPROXYRST target v%s options:\n"
"  --tproxyrst-mark value[/mask] Unset the value of the mark bits into nfmark\n"
"\n",
IPTABLES_VERSION);
}

static struct option tproxyrst_target_opts[] = {
	{ "tproxyrst-mark", 1, 0, '1' },
	{ 0 }
};

/* Initialize the target. */
static void
tproxyrst_target_init(struct ipt_entry_target *t, unsigned int *nfcache)
{
    /* NOTHING TODO */
}

static void
tproxyrst_target_final_check(unsigned int flags)
{
	if (!flags)
		exit_error(PARAMETER_PROBLEM,
		           "TPROXYRST target: Parameter --tproxyrst-mark"
                   " is required");
}

/* Function which parses command options; returns true if it
   ate an option */
static int
tproxyrst_target_parse(int c, char **argv, int invert, unsigned int *flags,
	 const struct ipt_entry *entry,
	 struct ipt_entry_target **target)
{
	struct ipt_tproxyrst_target_info *markinfo
		= (struct ipt_tproxyrst_target_info *)(*target)->data;
    char * end;

    switch(c)
    {
        case '1':
            if (*flags)
                exit_error(PARAMETER_PROBLEM,
                        "TPROXYRST target: Can't specify"
                        " --tproxyrst-mark twice");
#ifdef KERNEL_64_USERSPACE_32
            /* extract mark value from opt */
            markinfo->mark = strtoull(optarg, &end, 0);
            if (markinfo->mark > MAX_32BIT_VALUE)            
                exit_error(PARAMETER_PROBLEM, "Bad MARK value, "
                        "Should not be more than %d", MAX_32BIT_VALUE);

            /*  if mask value is given update with that the marsk feild if it
             *  was not mentioned update it with MAX_32BIT_VALUE to unset all
             *   bits mentioned in mark value*/
            if (*end == '/') {
                markinfo->mask = strtoull(end+1, &end, 0);
                if (markinfo->mask > MAX_32BIT_VALUE)
                    exit_error(PARAMETER_PROBLEM, "Bad MASK value, "
                        "Should not be more than %d", MAX_32BIT_VALUE);
            } else
                markinfo->mask = MAX_32BIT_VALUE;
#else
            markinfo->mark = strtoul(optarg, &end, 0);
            if (*end == '/') {
                markinfo->mask = strtoul(end+1, &end, 0);
            } else
                markinfo->mask = MAX_32BIT_VALUE;
#endif    
            if (*end != '\0' || end == optarg)
                exit_error(PARAMETER_PROBLEM, "Bad MARK value `%s'", optarg);
            
            /* set flags to 1 to avoid duplicate/multiple entries */ 
            *flags = 1;
            break;
            
        default:
            return 0;
    }

    return 1;
}

#ifdef KERNEL_64_USERSPACE_32
static void
print_mark(unsigned long long mark)
{
	printf("0x%llx", mark);
}
#else
static void
print_mark(unsigned long mark)
{
	printf("0x%lx", mark);
}
#endif

/* Prints out the targinfo. */
static void
tproxyrst_target_print(const struct ipt_ip *ip,
	 const struct ipt_entry_target *target,
	 int numeric)
{
	const struct ipt_tproxyrst_target_info *markinfo =
		(const struct ipt_tproxyrst_target_info *)target->data;
    
    printf("TPROXYRST mark ");
	print_mark(markinfo->mark);
	printf("/");
	print_mark(markinfo->mask);
}

/* Saves the union ipt_targinfo in parsable form to stdout. */
static void
tproxyrst_target_save(const struct ipt_ip *ip, const struct ipt_entry_target *target)
{
	const struct ipt_tproxyrst_target_info *markinfo =
		(const struct ipt_tproxyrst_target_info *)target->data;

    printf("--tproxyrst-mark ");
	print_mark(markinfo->mark);
	printf("/");
	print_mark(markinfo->mask);
}

/* TPROXY userspace target details */
static
struct iptables_target tproxyrst_target = {
	.next		= NULL,
	.name		= "TPROXYRST",
	.version	= IPTABLES_VERSION,
	.size		= IPT_ALIGN(sizeof(struct ipt_tproxyrst_target_info)),
	.userspacesize	= IPT_ALIGN(sizeof(struct ipt_tproxyrst_target_info)),
	.help		= &tproxyrst_target_help,
	.init		= &tproxyrst_target_init,
	.parse		= &tproxyrst_target_parse,
	.final_check	= &tproxyrst_target_final_check,
	.print		= &tproxyrst_target_print,
	.save		= &tproxyrst_target_save,
	.extra_opts	= tproxyrst_target_opts
};

void _init(void)
{
    /* register target with iptable's target */
	register_target(&tproxyrst_target);
}

